#!/usr/bin/ruby

require 'fileutils'
require 'json'
require 'date'
require 'csv'
require 'open-uri'

$output = []

$time_now_sec = Time.now.to_i

$hostname = `/bin/hostname -f`.chomp

ENV['PBSPRO_IGNORE_KERBEROS']='yes'

$all_nodes = {}

# all nodes per cluster
$all_nodes_cl = Hash.new(0)


$states = ['free', 'jobbusy', 'down', 'reserved', 'av_limited', 'maintenance', 'failure', 'blackhole', 'powersaving']

nodes_state = {}

$states.each do |state|
  nodes_state[state] = Hash.new(0)
end

cluster = {}
cluster['reserved'] = Hash.new(0)
cluster['maintenance'] = Hash.new(0)

$resources = ['cpu', 'gpu', 'mem', 'scratch']

node_res = {}
cl_res = {}
all_res = {}

$resources.each do |res|
   node_res[res] = {}
   node_res[res]['avail'] = Hash.new(0)
   node_res[res]['used'] = Hash.new(0)
   cl_res[res] = {}
   cl_res[res]['avail'] = Hash.new(0)
   cl_res[res]['used'] = Hash.new(0)
   all_res[res] = {}
   all_res[res]['avail'] = 0
   all_res[res]['used'] = 0
end

node_res['jobs'] = Hash.new(0)

nodes = JSON.parse(`/usr/bin/pbsnodes -a -F json`)

exit(0) if nodes['nodes'] == nil

nodes = nodes['nodes']

nodes.each do |node, attr| 
   # collect all nodes
  
   $all_nodes[node] = 1

   $states.each do |state|
     nodes_state[state][node] = 0
   end
   
   # node states - free, jobbosy, down, etc..
   if attr['state'] == 'free' and attr['jobs'] == nil
     nodes_state['free'][node] = 1
     if attr['resv'] != nil and attr['resv'] =~ /^M[0-9]/
       nodes_state['av_limited'][node] = 1
     end
   elsif attr['state'] == 'job-busy' or attr['jobs'] != nil
     nodes_state['jobbusy'][node] = 1
     if attr['resv'] != nil and attr['resv'] =~ /^M[0-9]/
       nodes_state['av_limited'][node] = 1
     end
   elsif attr['state'] =~ /resv-exclusive/
     nodes_state['reserved'][node] = 1
   elsif attr['state'] =~ /down/ or attr['state'] =~ /offline/
     nodes_state['down'][node] = 1
   else
     puts("Unknown state #{attr['state']} for node #{node}")
   end

   # number of jobs node is running
   if attr['jobs'] != nil
     node_res['jobs'][node] = attr['jobs'].size
   else
     node_res['jobs'][node] = 0
   end

   $resources.each do |res|
     node_res[res]['avail'][node] = 0
     node_res[res]['used'][node] = 0
   end

   # account cpu, gpu, mem, scratch resources
   if attr['resources_available']['ncpus'] != nil
     node_res['cpu']['avail'][node] = attr['resources_available']['ncpus']
   end
   if attr['resources_available']['ngpus'] != nil
     node_res['gpu']['avail'][node] = attr['resources_available']['ngpus']
   end
   if attr['resources_available']['mem'] != nil
     node_res['mem']['avail'][node] = attr['resources_available']['mem'].gsub('kb', '').to_i * 1024
   end
   if attr['resources_available']['scratch_local'] != nil 
     node_res['scratch']['avail'][node] = attr['resources_available']['scratch_local'].gsub('kb', '').to_i * 1024
   end
   if attr['resources_assigned']['ncpus'] != nil
     node_res['cpu']['used'][node] = attr['resources_assigned']['ncpus']
   end 
   if attr['resources_assigned']['ngpus'] != nil
     node_res['gpu']['used'][node] = attr['resources_assigned']['ngpus']
   end
   if attr['resources_assigned']['mem'] != nil
     node_res['mem']['used'][node] = attr['resources_assigned']['mem'].gsub('kb', '').to_i * 1024
   end
   scratch = 0
   if attr['resources_assigned']['scratch_local'] != nil
      scratch += attr['resources_assigned']['scratch_local'].gsub('kb', '').to_i * 1024
   end
   if attr['resources_assigned']['scratch_ssd'] != nil
      scratch += attr['resources_assigned']['scratch_ssd'].gsub('kb', '').to_i * 1024
   end
   node_res['scratch']['used'][node] = scratch

   if attr['queue'] == 'maintenance' 
      nodes_state['maintenance'][node] = 1
      nodes_state['down'][node] = 0
      nodes_state['free'][node] = 0
      nodes_state['jobbusy'][node] = 0
   end
   if attr['resv'] =~ /^R[0-9]*\./
      nodes_state['reserved'][node] = 1
      nodes_state['down'][node] = 0
      nodes_state['free'][node] = 0
      nodes_state['jobbusy'][node] = 0
   end
   if attr['resv'] =~ /^M[0-9]*\./
      nodes_state['maintenance'][node] = 1
      nodes_state['down'][node] = 0
      nodes_state['free'][node] = 0
      nodes_state['jobbusy'][node] = 0
   end
end

$all_nodes.each do |node, v|
  clustername = node.gsub(/[0-9]*/, '')

  if nodes_state['maintenance'][node] == 1 and nodes_state['reserved'][node] = 1
     nodes_state['reserved'][node] = 0
  end

  if nodes_state['reserved'][node] == 1
    cpu_used = node_res['cpu']['avail'][node]
    gpu_used = node_res['gpu']['avail'][node]
    mem_used = node_res['mem']['avail'][node]
    scratch_used = node_res['scratch']['avail'][node]
  elsif nodes_state['maintenance'][node] == 1
    cpu_used = 0
    gpu_used = 0
    mem_used = 0
    scratch_used = 0
  else
    cpu_used = node_res['cpu']['used'][node]
    gpu_used = node_res['gpu']['used'][node]
    mem_used = node_res['mem']['used'][node]
    scratch_used = node_res['scratch']['used'][node]
  end

  $output << { 'name'          => 'av_state',
               'free'          => nodes_state['free'][node],
               'jobbusy'       => nodes_state['jobbusy'][node],
               'down'          => nodes_state['down'][node],
               'reserved'      => nodes_state['reserved'][node],
               'av_limited'    => nodes_state['av_limited'][node],
               'maintenance'   => nodes_state['maintenance'][node],
               'failure'       => nodes_state['failure'][node],
               'blackhole'     => nodes_state['blackhole'][node],
               'powersaving'   => nodes_state['powersaving'][node],
               'cpu_avail'     => node_res['cpu']['avail'][node],
               'cpu_used'      => cpu_used,
               'gpu_avail'     => node_res['gpu']['avail'][node],
               'gpu_used'      => gpu_used,
	       'mem_avail'     => node_res['mem']['avail'][node],
               'mem_used'      => mem_used,
	       'scratch_avail' => node_res['scratch']['avail'][node],
               'scratch_used'  => scratch_used,
               'node'          => node,
               'cluster'       => clustername,
               'host'          => $hostname,
               'time'          => $time_now_sec}
end

puts(JSON.pretty_generate($output))
